#include <string.h>

#include "matrix_buffer.h"

/**
 * @brief Attempt to read a message size stored in a row.
 *
 * @param row.
 * @param size, if stores, zero otherwise.
 * @returns true, if value was succesfully read.
 */
bool protocol_get_message_size(MatrixRow* row, uint16_t* size)
{
	if (row->size < 2)
	{
		size = 0;

		return false;
	}

	*size = (row->data[0] << 8) + row->data[1];

	return true;
}

/**
 * @brief Check if the whole message is in the row. Function
 *        assumes that all bytes were read correctly and
 *        the size in the datagram is correct.
 *
 * @param row.
 * @returns true, if the whole message is in the row.
 */
bool protocol_message_ready(MatrixRow* row)
{
	uint16_t message_size = 0;

	if (!protocol_get_message_size(row, &message_size))
	{
		return false;
	}

	return row->size == (2 + message_size + 1);
}


/**
 * @brief Compute checksum from a given data.
 *
 * @param data.
 * @param size of the data.
 * @param checksum.
 */
void protocol_calculate_checksum(uint8_t* data, uint32_t size, uint8_t* checksum)
{
	*checksum = 0;

	for (uint32_t i = 0; i < size; i++)
	{
		*checksum ^= data[i];
	}
}

/**
 * @brief Try to get a stored checksum from the datagram.
 *
 * @param row.
 * @param stored checksum, or zero is datagram is incomplete.
 * @returns true if checksum was read and returned, false otherwise.
 */
bool protocol_get_checksum(MatrixRow* row, uint8_t* checksum)
{
	uint16_t message_size = 0;
	*checksum = 0;

	if (!protocol_get_message_size(row, &message_size))
	{
		return false;
	}

	if (row->size < 2 + message_size + 1)
	{
		return false;
	}

	*checksum = row->data[message_size + 2];

	return true;
}

/**
 * @brief Attempt to copy a message from the datagram into a data param.
 *		  In case of an error, function returns false and won't change
 *		  the data buffer. Ensure that buffer is large enough.
 *
 * @param row.
 * @param buffer.
 * @param size of the buffer.
 * @returns true if message was copied successfully, false otherwise.
 */
bool protocol_get_inner_message(MatrixRow* row, uint8_t* data, uint32_t size)
{
	uint16_t message_size = 0;

	if (!protocol_get_message_size(row, &message_size))
	{
		return false;
	}

	if (row->size < 2 + message_size || message_size <= size)
	{
		return false;
	}

	memcpy(data, row->data, size);

	return true;
}
